@@ -28,7 +28,7 @@ import gtk, gnome
 import abstract, idbyname, const, gu, statisticsviewer, widgets
 import soundcard, mpd, mpd.musicdisplayer
 import utils
-import dataparser
+import lessonfile
 
 class Teacher(idbyname.Teacher):
     exercise_data = __exercise_data__
@@ -48,7 +48,6 @@ class Gui(abstract.LessonbasedGui):
         # practise_box #
         ################
         self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
-        self.g_music_displayer.set_size_request(-1, 170)
         self.practise_box.pack_start(self.g_music_displayer)
 
         self.g_flashbar = gu.FlashBar()
@@ -114,24 +113,22 @@ class Gui(abstract.LessonbasedGui):
             self.g_music_displayer.display(self.m_t.m_P.get_music(),
                                self.get_int('config/feta_font_size=20'))
     def new_question(self, widget=None):
+        def exception_cleanup():
+            soundcard.synth.stop()
+            self.g_repeat.set_sensitive(False)
+            self.g_show.set_sensitive(False)
+            self.g_guess_answer.set_sensitive(False)
+            self.g_entry.set_text("")
+            self.g_music_displayer.clear(2)
         if not self.m_t.m_P:
             return
         # pop just in case there is something in the stack.
         self.g_flashbar.pop()
-        if self.m_t.new_question() == self.m_t.OK:
-            try:
+        try:
+            g = self.m_t.new_question()
+            if g == self.m_t.OK:
                 self.g_music_displayer.display(self.m_t.m_P.get_music(),
                             self.get_int('config/feta_font_size=20'), mpd.FIRST)
-            except mpd.MpdException, e:
-                soundcard.synth.stop()
-                self.g_win.display_question_music_error_message(
-                   self.m_t.m_P._idx, self.get_string('lessonfile'), e)
-                self.g_repeat.set_sensitive(False)
-                self.g_show.set_sensitive(False)
-                self.g_guess_answer.set_sensitive(False)
-                self.g_entry.set_text("")
-                self.g_music_displayer.clear()
-            else:
                 self.g_repeat.set_sensitive(True)
                 self.g_show.set_sensitive(True)
                 self.g_guess_answer.set_sensitive(True)
@@ -140,6 +137,13 @@ class Gui(abstract.LessonbasedGui):
                      not self.get_bool('config/picky_on_new_question'))
                 self.m_t.play_question()
                 self.g_entry.grab_focus()
+        except mpd.MpdException, e:
+            exception_cleanup()
+            self.g_win.display_question_music_error_message(
+               self.m_t.m_P._idx, self.get_string('lessonfile'), e, __file__)
+        except lessonfile.LessonfileException, e:
+            exception_cleanup()
+            self.g_win.display_exception_message(e, __file__)
     def give_up(self, widget=None):
         self.m_t.give_up()
         self.g_new.set_sensitive(True)
@@ -156,10 +160,10 @@ class Gui(abstract.LessonbasedGui):
         self.g_repeat.set_sensitive(False)
         self.g_guess_answer.set_sensitive(False)
         self.g_show.set_sensitive(False)
-        self.g_music_displayer.clear()
+        self.g_music_displayer.clear(2)
         self.m_t.end_practise()
     def update_gui_after_lessonfile_change(self):
-        self.g_music_displayer.clear()
+        self.g_music_displayer.clear(2)
         if self.m_t.m_P:
             self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
             self.g_new.set_sensitive(True)
