@@ -24,12 +24,29 @@ except:
     import i18n
     i18n.setup_srcdir()
 
-class NoQuestionException(Exception):
-    pass
 class LessonfileException(Exception):
-    pass
+    def get_title(self):
+        return "LessonfileException"
 
-import re, string, types, os, sys
+class NoQuestionInFileException(LessonfileException):
+    """
+    Raised by find_random_question if the lesson file contains
+    no questions at all.
+    """
+    def __init__(self, lessonfilename):
+        self.m_filename = lessonfilename
+    def __str__(self):
+        return "The lesson file '%s' contains no questions." % self.m_filename
+
+class NoQuestionConfiguredException(LessonfileException):
+    """
+    This exception is raised by select_random_question if the user has
+    unselected all the questions available in the lesson file.
+    """
+    def __str__(self):
+        return "All questions are unselected. Select some questions from the config page."
+
+import re, os, sys
 import const
 import random
 import mpd
@@ -76,6 +93,9 @@ predef = {'dictation': 'dictation',
           'chord-voicing': 'chord-voicing',
           'chord': 'chord',
           'id-by-name': 'id-by-name',
+          'normal': 'normal',
+          'voice': 'voice',
+          'rvoice': 'rvoice',
           'satb': 'satb',
           'cmdline': 'cmdline',
           'horiz': 'horiz',
@@ -97,6 +117,7 @@ lessonfile_functions = {
         '_': dataparser.dataparser_i18n_func,
         '_i': _i, #FIXME
         'play_wav': lambda f: WavPlayerFunctionClass(f),
+        'music': lambda m, t='normal': Music(m, t),
 }
 
 class _Header:
@@ -114,15 +135,22 @@ class _Header:
                            ('filldir', 'horiz'),
                            ('questions_selectable', False),
                            ('have_repeat_slowly_button', False),
+                           ('have_repeat_arpeggio_button', False),
                            ('have_music_displayer', False),
+                           ('enable_right_click', True),
                            ):
-            if not self.m_headerdict.has_key(key):
+            if key not in self.m_headerdict:
                 self.m_headerdict[key] = value
     def __getattr__(self, name):
-        if self.m_headerdict.has_key(name):
+        if name in self.m_headerdict:
             return dataparser.get_translated_string(self.m_headerdict, name)
         raise NameError(name)
 
+class Music:
+    def __init__(self, musicdata, musictype='normal'):
+        self.m_musicdata = musicdata
+        self.m_musictype = musictype
+
 
 class LessonfileCommon(object):
     def __init__(self, teacher):
@@ -143,6 +171,9 @@ class LessonfileCommon(object):
         self.m_questions = parser.questions
         for question in self.m_questions:
             question['active'] = 1
+            # FIXMECOMPAT
+            if isinstance(question['music'], basestring):
+                question['music'] = Music(question['music'], self.header.musicformat)
             question['m_lessonfile'] = self
     def select_random_question(self):
         """
@@ -150,11 +181,13 @@ class LessonfileCommon(object):
         """
         count = 0
         available_question_idx = []
+        if not self.m_questions:
+            raise NoQuestionInFileException(self.m_filename)
         for i in range(len(self.m_questions)):
             if self.m_questions[i]['active']:
                 available_question_idx.append(i)
         if not available_question_idx:
-            raise NoQuestionException()
+            raise NoQuestionConfiguredException()
         while 1:
             count += 1
             self._idx = random.choice(available_question_idx)
@@ -162,16 +195,18 @@ class LessonfileCommon(object):
                 if self.header.random_transpose == True:
                     self.handle_random_transpose_is_yes()
                 # Backward compatability to handle old style
-                # random_transpose = -4, 5 FIXME
+                # random_transpose = -4, 5 FIXMECOMPAT
                 elif len(self.header.random_transpose) == 2:
                     self.header.random_transpose \
                         = ['semitones'] + self.header.random_transpose
-                if self.m_questions[self._idx].has_key('key'):
+                if 'key' in self.m_questions[self._idx]:
                     key = self.m_questions[self._idx]['key']
                 else:
                     key = "c \major"
                 if self.header.random_transpose[0] == 'semitones':
                     self.m_transpose = self.semitone_find_random_transpose()
+                    if random.randint(0, 1):
+                        self.m_transpose.enharmonic_flip()
                 else:
                     self.m_transpose = self.find_random_transpose(key)
             if count == 10:
@@ -187,20 +222,19 @@ class LessonfileCommon(object):
         Create and return a random MusicalPitch representing this transposition.
         """
         assert self.header.random_transpose[0] == 'semitones'
-        if self.header.random_transpose == 1:
-            return mpd.MusicalPitch().randomize("g", "fis'")
-        elif type(self.header.random_transpose) in (type([]), type(())):
-            return mpd.MusicalPitch().randomize(
+        return mpd.MusicalPitch().randomize(
               mpd.transpose_notename("c'", self.header.random_transpose[1]),
               mpd.transpose_notename("c'", self.header.random_transpose[2]))
     def find_random_transpose(self, key):
         """
         Called to create random transposition in "accidentals" or "key" mode.
         Create and return a random MusicalPitch representing this transposition.
+        Keyword arguments:
+        key -- the key the question is written in, for example "c \major"
         """
         assert self.header.random_transpose[0] in ('key', 'accidentals')
         low, high = self.header.random_transpose[1:3]
-        tone, minmaj = string.split(key, " ")
+        tone, minmaj = key.split()
         k = mpd.MusicalPitch.new_from_notename(tone).get_octave_notename()
         #FIXME this list say what key signatures are allowed in sing-chord
         # lesson files. Get the correct values and document them.
@@ -224,7 +258,7 @@ class LessonfileCommon(object):
         interv.set_from_string(_keys_to_interval[random.choice(range(n_down, n_up+1))])
         return mpd.MusicalPitch.new_from_notename("c'") + interv
     def handle_random_transpose_is_yes(self):
-        self.header.random_transpose = ['semitones', -5, 6]
+        self.header.random_transpose = ['key', -5, 5]
     def iterate_questions_with_unique_names(self):
         """Iterate the questions in the lessonfile, but only yield the
         first question if several questions have the same name. The
@@ -232,7 +266,7 @@ class LessonfileCommon(object):
         """
         names = {}
         for question in self.m_questions:
-            if not names.has_key(question.get_cname()):
+            if question.get_cname() not in names:
                 names[question.get_cname()] = 1
                 yield question
     def get_unique_cnames(self):
@@ -268,42 +302,43 @@ class LessonfileCommon(object):
         """
         assert self._idx is not None
         return self.m_questions[self._idx].get_cname()
-    def get_music_of_question(self, question):
-        """
-        Return the music of question 'question'. This is complete
-        music code that can be fed to mpd.play_music(...).
-        """
-        return self.prepare_music_from_string('music', question)
-    def prepare_music_from_string(self, name, question):
-        """
-        """
-        if self.header.musicformat == 'chord':
+    def prepare_music_from_string(self, question, varname):
+        if question[varname].m_musictype == 'chord':
             return r"\staff\transpose %s{< %s > }" \
-              % (self.m_transpose.get_octave_notename(), question[name])
-        elif self.header.musicformat == 'normal':
-            return self.prepare_music_string_normal_format(question[name])
-        elif self.header.musicformat == 'satb':
-            return self.prepare_music_string_satb_format(question[name], question)
-        elif self.header.musicformat == 'cmdline':
-            return str(question[name])
+              % (self.m_transpose.get_octave_notename(), question[varname].m_musicdata)
+        elif question[varname].m_musictype == 'normal':
+            return self.prepare_music_string_normal_format(question, varname)
+        elif question[varname].m_musictype == 'satb':
+            return self.prepare_music_string_satb_format(question, varname)
+        elif question[varname].m_musictype == 'voice':
+            return r"\staff\transpose %s{ %s }" \
+              % (self.m_transpose.get_octave_notename(), question[varname].m_musicdata)
+        elif question[varname].m_musictype == 'rvoice':
+            ns = question[varname].m_musicdata.split()[0]
+            rel_note = re.match("[a-zA-Z]+[',]*[\d]*\.*", ns).group()
+            return r"\staff\transpose %s\relative c'{ %s }" \
+              % (self.m_transpose.get_octave_notename(), question[varname].m_musicdata)
+        elif question[varname].m_musictype == 'cmdline':
+            return str(question[varname])
         else:
-            raise LessonfileException("\nLesson file contain illegal assignment 'musicformat=%s'" % self.header.musicformat)
+            raise LessonfileException("\nUknown music type: '%s'" % question[varname].m_musictype)
     def get_music(self):
         """
         Return the music for the currently selected question. This is complete
         music code that can be fed to mpd.play_music(...).
+        FIXMEFIXME Currently this returns the MUSIC variable.
         """
         assert self._idx is not None
-        return self.get_music_of_question(self.m_questions[self._idx])
-    def prepare_music_string_satb_format(self, s, question):
+        return self.prepare_music_from_string(self.m_questions[self._idx], 'music')
+    def prepare_music_string_satb_format(self, question, varname):
         """
         It should not be necessary to call this method directly. Call
         .get_music() or .prepare_music_from_string() and let that
         function decide.
         """
-        assert self.header.musicformat == 'satb'
-        v = string.split(s, '|')
-        if question.has_key('key'):
+        assert question[varname].m_musictype == 'satb'
+        v = question[varname].m_musicdata.split('|')
+        if 'key' in question:
             k = question['key']
         else:
             k = "c \major"
@@ -314,103 +349,107 @@ class LessonfileCommon(object):
                 \addvoice{ \stemDown <%s>}
                 """ % (k, v[0], v[1], k, v[2], v[3])
         if self.header.random_transpose:
-            music = string.replace(music, r"\staff",
+            music = music.replace(r"\staff",
                       r"\staff\transpose %s" % self.m_transpose.get_octave_notename())
-            music = string.replace(music, r"\addvoice",
+            music = music.replace(r"\addvoice",
                       r"\addvoice\transpose %s" % self.m_transpose.get_octave_notename())
         return music
-    def prepare_music_string_normal_format(self, s):
+    def prepare_music_string_normal_format(self, question, varname):
         """
         It should not be necessary to call this method directly. Call
         .get_music() or .prepare_music_from_string() and let that
         function decide.
         """
-        assert self.header.musicformat == 'normal'
+        assert question[varname].m_musictype == 'normal'
         if self.header.random_transpose:
-            s = string.replace(s, r'\staff',
+            s = question[varname].m_musicdata.replace(r'\staff',
                r'\staff\transpose %s' % self.m_transpose.get_octave_notename())
-            s = string.replace(s, r'\addvoice',
+            s = s.replace(r'\addvoice',
                r'\addvoice\transpose %s' % self.m_transpose.get_octave_notename())
-        return s
-    def get_music_as_notename_list(self):
+            return s
+        return question[varname].m_musicdata
+    def get_music_as_notename_list(self, varname):
         """
-        Return a list of notenames representing the question currently
-        selected question
+        Return a list of notenames from the variabale VARNAME in the
+        currently selected question. The notes are transposed if
+        header.random_transpose is set.
 
-        Assert musicformat in ('chord', 'satb')
+        Assert m_musictype in ('chord', 'satb')
         """
-        assert self.header.musicformat in ('chord', 'satb')
         assert self._idx is not None
-        if self.header.musicformat == 'chord':
+        music = self.get_question()[varname]
+        assert music.m_musictype in ('chord', 'satb')
+        if music.m_musictype == 'chord':
             if not self.header.random_transpose:
-                return string.split(self.m_questions[self._idx]['music'])
+                return music.m_musicdata.split()
             v = []
-            for n in string.split(self.m_questions[self._idx]['music']):
+            for n in music.m_musicdata.split():
                 v.append(mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(
                      self.m_transpose).get_octave_notename())
             return v
-        elif self.header.musicformat == 'satb':
+        elif music.m_musictype == 'satb':
             v = []
-            for n in re.split("\s|\|", self.m_questions[self._idx]['music']):
+            for n in re.split("\s|\|", music.m_musicdata):
                 v.append(mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(self.m_transpose).get_octave_notename())
             return v
         else:
-            raise LessonfileException("get_music_as_notename_list cannot handle musicformat=%s" % self.header.musicformat)
-    def get_music_as_notename_string(self):
+            raise LessonfileException("get_music_as_notename_list cannot handle m_musictype==%s" % music.m_musictype)
+    def get_music_as_notename_string(self, varname):
         """
         Return a string with notenames representing the question currently
-        selected question.
+        selected question. The notes are transposed if
+        header.random_transpose is set.
 
         Assert musicformat in ('chord', 'satb')
         """
-        return " ".join(self.get_music_as_notename_list())
+        return " ".join(self.get_music_as_notename_list(varname))
     def has_question(self):
         """
         Return True if a question is selected.
         """
-        return self._idx != None
-    def play_question(self, question=None):
+        return self._idx is not None
+    def play_question(self, question=None, varname='music'):
         """Play the question. Play the current question if question is none.
+        varname is the name of the variable that contains the music.
         """
         if not question:
             question = self.get_question()
-        if self.header.musicformat == 'cmdline':
-            app.run_external_program(self.get_music(), cfg.get_string('lessoncollections/%s' % self.m_t.get_string('lessoncollection')), "")
-            return
-        elif self.header.musicformat in ('normal', 'satb'):
+        if question[varname].m_musictype == 'cmdline':
+            app.run_external_program(str(question[varname].m_musicdata), cfg.get_string('lessoncollections/%s' % self.m_t.get_string('lessoncollection')), "")
+        elif question[varname].m_musictype in ('normal', 'satb', 'voice', 'rvoice'):
             self.play_question_musicformat_normal_satb(question)
-            return
-        elif self.header.musicformat == 'chord':
+        elif question[varname].m_musictype in 'chord':
             self.play_question_musicformat_chord(question)
-            return
-        raise FIXME
+        else:
+            raise LessonfileException("Unrecognised musictype '%s'" % question[varname].m_musictype)
     def play_question_musicformat_normal_satb(self, question):
-        assert self.header.musicformat in ('normal', 'satb')
         instrument = self.prepare_instrument_list(question)
         mpd.play_music(question.get_music(), self.get_tempo(),
                        instrument[0], instrument[1])
     def play_question_slowly(self, question=None):
         if not question:
             question = self.get_question()
-        assert self.header.musicformat == 'normal'
+        assert question['music'].m_musictype == 'normal'
         self.play_question_musicformat_normal_slowly(question)
     def play_question_musicformat_normal_slowly(self, question):
-        assert self.header.musicformat == 'normal'
+        assert question['music'].m_musictype == 'normal'
         instrument = self.prepare_instrument_list(question)
         bpm = self.get_tempo()
         bpm = (bpm[0]/2, bpm[1])
         mpd.play_music(question.get_music(), bpm,
                        instrument[0], instrument[1])
     def play_question_musicformat_chord(self, question):
-        assert self.header.musicformat == 'chord'
+        assert question['music'].m_musictype == 'chord'
         instrument = self.prepare_instrument_list(question)
+        assert len(instrument) in (2, 6)
         if len(instrument) == 2:
             mpd.play_music(question.get_music(), self.get_tempo(),
                        instrument[0], instrument[1])
-        elif len(instrument) == 6:
+        else:
+            assert len(instrument) == 6
             track = soundcard.Track()
             track.set_bpm(cfg.get_int('config/default_bpm'))#FIXME skulle dette vart tatt fra lesson file?
-            nlist = self.get_music_as_notename_list()
+            nlist = self.get_music_as_notename_list(varname)
             # set patches
             track.set_patch(0, instrument[0])
             for x in range(1, len(nlist)-1):
@@ -431,28 +470,29 @@ class LessonfileCommon(object):
                                    instrument[3])
             track.stop_note(len(nlist)-1, mpd.notename_to_int(nlist[-1]), instrument[5])
             soundcard.synth.play_track(track)
+    def play_question_arpeggio(self, varname='music'):
+        musictype = self.get_question()[varname].m_musictype
+        if musictype == 'chord':
+            self.play_question_musicformat_chord_arpeggio(varname)
+        elif musictype == 'satb':
+            self.play_question_musicformat_satb_arpeggio(varname)
         else:
-            raise "FIXME"
-    def play_question_arpeggio(self):
-        if self.header.musicformat == 'chord':
-            self.play_question_musicformat_chord_arpeggio()
-        elif self.header.musicformat == 'satb':
-            self.play_question_musicformat_satb_arpeggio()
-        else:
-            raise FIXME
-    def play_question_musicformat_chord_arpeggio(self):
+            raise LessonfileException("\nUknown music type: '%s'" % self.get_question()[varname].m_musictype)
+    def play_question_musicformat_chord_arpeggio(self, varname):
         """FIXME -- for completeness, should this function should take a question argument?
         """
-        assert self.header.musicformat == 'chord'
+        assert self.get_question()[varname].m_musictype == 'chord'
         instrument = self.prepare_instrument_list(self.get_question())
+        assert len(instrument) in (2, 6)
         if len(instrument) == 2:
-            m = self.get_music_as_notename_string()
+            m = self.get_music_as_notename_string(varname)
             mpd.play_music(r"\staff{%s}" % m, self.get_tempo(),
                        instrument[0], instrument[1])
-        elif len(instrument) == 6:
+        else:
+            assert len(instrument) == 6
             track = soundcard.Track()
             track.set_bpm(cfg.get_int('config/default_bpm'))#FIXME skulle dette vart tatt fra lesson file?
-            nlist = self.get_music_as_notename_list()
+            nlist = self.get_music_as_notename_list(varname)
             # set patches
             track.set_patch(0, instrument[0])
             for x in range(1, len(nlist)-1):
@@ -465,26 +505,25 @@ class LessonfileCommon(object):
                                     instrument[3])
             track.note(4, len(nlist)-1, mpd.notename_to_int(nlist[-1]), instrument[5])
             soundcard.synth.play_track(track)
-        else:
-            raise "FIXME"
     def play_question_musicformat_satb_arpeggio(self):
         """FIXME -- for completeness, should this function should take a question argument?
         """
+        varname = 'music'
         instrument = self.prepare_instrument_list(self.get_question())
         track = soundcard.Track()
         track.set_bpm(cfg.get_int('config/default_bpm'))
         track.set_patch(0, instrument[0])
         for x in [0, 1]:
-            v = self.get_music_as_notename_list()[x]
-            s = string.split(string.strip(v), " ")
+            v = self.get_music_as_notename_list(varname)[x]
+            s = v.strip().split(" ")
             for n in s:
                 if cfg.get_string('user/sex') == 'female':
                     track.note(4, 0, mpd.notename_to_int(n), instrument[1])
                 else:
                     track.note(4, 0, mpd.notename_to_int(n)-12, instrument[1])
         for x in [2, 3]:
-            v = self.get_music_as_notename_list()[x]
-            s = string.split(string.strip(v), " ")
+            v = self.get_music_as_notename_list(varname)[x]
+            s = v.strip().split(" ")
             for n in s:
                 if cfg.get_string('user/sex') == 'male':
                     track.note(4, 0, mpd.notename_to_int(n), instrument[1])
@@ -514,28 +553,30 @@ class LessonfileCommon(object):
         Use app default values if the variable is missing.
         Convert instrument names to integer values.
         """
-        if question.has_key('instrument'):
+        if 'instrument' in question:
             instrument = question['instrument']
-        elif self.m_globals.has_key('instrument'):
+        elif 'instrument' in self.m_globals:
             instrument = self.m_globals['instrument']
         else:
             instrument = [cfg.get_int('config/preferred_instrument'),
                           cfg.get_int('config/preferred_instrument_velocity')]
-        if type(instrument) in (types.UnicodeType, types.IntType):
+        if isinstance(instrument, (unicode, int)):
             instrument = [instrument,
                           cfg.get_int('config/preferred_instrument_velocity')]
+        assert len(instrument) in (2, 6)
         if len(instrument) == 2:
-            if type(instrument[0]) == types.UnicodeType:
+            if isinstance(instrument[0], unicode):
                 try:
                     instrument[0] = soundcard.find_midi_instrument_number(instrument[0])
                 except KeyError, e:
-                    print "Warning: Invalid instrument name in lesson file:", e
+                    print >> sys.stderr, "Warning: Invalid instrument name '%s' in lesson file:" % instrument[0], e
                     instrument[0] = cfg.get_int('config/preferred_instrument')
             if not (0 <= instrument[1] < 128):
+                print >> sys.stderr, "Warning: Adjusting instrument velocity since this value is invalid '%s'" % instrument[1]
                 instrument[1] = cfg.get_int('config/preferred_instrument_velocity')
         elif len(instrument) == 6:
             for x in (0, 2, 4):
-                if type(instrument[x]) == types.UnicodeType:
+                if isinstance(instrument[x], unicode):
                     try:
                         instrument[x] = soundcard.find_midi_instrument_number(instrument[x])
                     except KeyError, e:
@@ -543,8 +584,6 @@ class LessonfileCommon(object):
                         instrument[0] = cfg.get_int('config/preferred_instrument')
                 if not (0 <= instrument[x+1] < 128):
                     instrument[x+1] = cfg.get_int('config/preferred_instrument_velocity')
-        else:
-            raise "FIXME"
         return instrument
 
 
@@ -555,7 +594,7 @@ class DictationLessonfile(LessonfileCommon):
     def get_breakpoints(self):
         assert self._idx is not None
         r = []
-        if self.m_questions[self._idx].has_key('breakpoints'):
+        if 'breakpoints' in self.m_questions[self._idx]:
             r = self.m_questions[self._idx]['breakpoints']
             if not type(r) == type([]):
                 r = [r]
@@ -563,11 +602,11 @@ class DictationLessonfile(LessonfileCommon):
         return r
     def get_clue_end(self):
         assert self._idx is not None
-        if self.m_questions[self._idx].has_key('clue_end'):
-            return apply(Rat, self.m_questions[self._idx]['clue_end'])
+        if 'clue_end' in self.m_questions[self._idx]:
+            return Rat(*self.m_questions[self._idx]['clue_end'])
     def get_clue_music(self):
         assert self._idx is not None
-        if self.m_questions[self._idx].has_key('clue_music'):
+        if 'clue_music' in self.m_questions[self._idx]:
             return self.m_questions[self._idx]['clue_music']
     def select_previous(self):
         """
@@ -605,7 +644,7 @@ class IdByNameLessonfile(LessonfileCommon):
     def parse_file(self, filename):
         super(IdByNameLessonfile, self).parse_file(filename)
         for question in self.m_questions:
-            if not question.has_key('name'):
+            if 'name' not in question:
                 print >> sys.stderr, "IdByNameLessonfile.parse_file: discarding question in the lessonfile '%s'\nbecause of missing 'name' variable" % filename
                 continue
 
@@ -640,11 +679,11 @@ class ChordLessonfile(LessonfileCommon):
             self.m_chord_types[question.get_cname()] = question.get_name()
 
             # inversion
-            if question.has_key('inversion') \
+            if 'inversion' in question \
                     and question['inversion'] not in self.m_inversions:
                 self.m_inversions.append(question['inversion'])
             # toptone
-            if question.has_key('toptone') \
+            if 'toptone' in question \
                     and question['toptone'] not in self.m_toptones:
                 self.m_toptones.append(question['toptone'])
     def get_inversion(self):
@@ -654,8 +693,8 @@ class ChordLessonfile(LessonfileCommon):
         Return -1 if no inversion info is set in the question.
         """
         assert self._idx is not None
-        if self.m_questions[self._idx].has_key('inversion'):
-             return self.m_questions[self._idx]['inversion']
+        if 'inversion' in self.m_questions[self._idx]:
+            return self.m_questions[self._idx]['inversion']
         return -1
     def get_toptone(self):
         """
@@ -663,7 +702,7 @@ class ChordLessonfile(LessonfileCommon):
         Return -1 if no toptone info is set in the question.
         """
         assert self._idx is not None
-        if self.m_questions[self._idx].has_key('toptone'):
+        if 'toptone' in self.m_questions[self._idx]:
             return self.m_questions[self._idx]['toptone']
         return -1
 
@@ -674,7 +713,7 @@ class LessonFileManager:
         self.m_db = {}
         self.m_invalid_files = []
         vim_tmpfile_re = re.compile("\..*\.swp")
-        for collection in string.split(cfg.get_string("config/lessoncollections")):
+        for collection in cfg.get_list("config/lessoncollections"):
             self.m_db[collection] = {}
             if not os.path.isdir(os.path.expanduser(cfg.get_string("lessoncollections/%s" % collection))):
                 continue
@@ -690,25 +729,25 @@ class LessonFileManager:
                     p.parse_file(self.get_complete_filename(collection, filename))
                 except dataparser.DataparserException, e:
                     self.m_invalid_files.append((collection, filename))
-                    print >> sys.stderr, "Discarding lessonfile '%s/%s' because the file contain errors:" % (collection, filename)
+                    print >> sys.stderr, "LessonFileManager.parse: Discarding lessonfile '%s/%s' because the file contain errors:" % (collection, filename)
                     print >> sys.stderr, str(e)
                     continue
                 c = p.header['content']
-                if type(c) in types.StringTypes:
+                if isinstance(c, basestring):
                     c = [c]
                 for u in c:
-                    if not self.m_db[collection].has_key(u):
+                    if u not in self.m_db[collection]:
                         self.m_db[collection][u] = []
-                    if p.header.has_key('title'):
+                    if 'title' in p.header:
                         title = p.header['title']
                     else:
                         title = ""
                     self.m_db[collection][u].append({'filename': filename,
                                                      'title': title}
                                                 )
-        for collection in self.m_db.keys():
-            for content in self.m_db[collection].keys():
-                self.m_db[collection][content].sort()
+        for collection in self.m_db.itervalues():
+            for content in collection:
+                collection[content].sort()
         self.m_htmldoc = """<html><body><p>This list is updated each time
                 you start Solfege. As you can see, Solfege need more
                 lessonfiles. <a href="lessonfiles.html">Write one!</a>"""
@@ -717,9 +756,9 @@ class LessonFileManager:
             for collection, filename in self.m_invalid_files:
                 self.m_htmldoc += "<li>%(filename)s in collection %(collection)s</li>" % {'collection': collection, 'filename': filename}
             self.m_htmldoc += "</ul>"
-        for collection in self.m_db.keys():
+        for collection in self.m_db:
             self.m_htmldoc = self.m_htmldoc + "<h1>collection: %s</h1>" % collection
-            for use in self.m_db[collection].keys():
+            for use in self.m_db[collection]:
                 self.m_htmldoc = self.m_htmldoc + "<h2>%s</h2><p>" % use
                 for d in self.m_db[collection][use]:
                     fn = d['filename']
@@ -741,7 +780,7 @@ class LessonFileManager:
         return os.path.expanduser(os.path.join(
          cfg.get_string('lessoncollections/%s' % catalog), fn))
     def get_filenames(self, collection, usetype):
-        if self.m_db[collection].has_key(usetype):
+        if usetype in self.m_db[collection]:
             v = []
             for d in self.m_db[collection][usetype]:
                 v.append(d['filename'])
@@ -749,20 +788,26 @@ class LessonFileManager:
         else:
             return []
     def get_info(self, collection, usetype):
-        if self.m_db[collection].has_key(usetype):
+        if usetype in self.m_db[collection]:
             return self.m_db[collection][usetype]
         else:
             return []
 
 if __name__ == '__main__':
     import time
+    t1 = time.clock()
     cfg.initialise(None, "default.config",
-        "~/.solfegerc1.9")
-    t1=time.clock()
-    for n in range(10):
+        "~/.solfegerc")
+    t2 = time.clock()
+    for n in range(1):
         #p = LessonfileCommon()
         #p.parse_file(sys.argv[1])
-        p = LessonFileManager()
-    t2 = time.clock()
-    print t2-t1
-    print _
+        #p = LessonFileManager()
+        p = LessonfileCommon(None)
+        p.parse_file(sys.argv[1])
+        print p.m_globals
+        for q in p.m_questions:
+            print q
+    t3 = time.clock()
+    #print t3-t2
+    #print t2-t1
