@@ -27,7 +27,6 @@ exercise_setup.register_exercise(__exercise_data__)
 import gtk, gnome.ui
 import abstract, const, gu, mpd, soundcard
 import random, os
-import string
 import cfg
 
 RHYTHMS = ("c4", "c8 c8", "c16 c16 c16 c16", "c8 c16 c16",
@@ -60,7 +59,7 @@ class Teacher(abstract.Teacher):
 
         if self.get_bool('config/picky_on_new_question') \
                  and self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]:
-             return Teacher.ERR_PICKY
+            return Teacher.ERR_PICKY
 
         self.q_status = const.QSTATUS_NO
 
@@ -101,7 +100,7 @@ class Teacher(abstract.Teacher):
         assert self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]
         v = []
         for idx in range(len(self.m_question)):
-           v.append(self.m_question[idx] == a[idx])
+            v.append(self.m_question[idx] == a[idx])
         if filter(lambda a: a == 0, v) == []:
             self.q_status = const.QSTATUS_SOLVED
             self.maybe_auto_new_question()
@@ -263,6 +262,7 @@ class Gui(abstract.Gui):
         self.practise_box.pack_start(gtk.HBox(), False,
                                      padding=gnome.ui.PAD_SMALL)
         self.g_rhythm_viewer = RhythmViewer(self)
+        #FIXME the value 52 is dependant on the theme used
         self.g_rhythm_viewer.set_size_request(-1, 52)
         self.g_rhythm_viewer.create_holders()
         self.practise_box.pack_start(self.g_rhythm_viewer, False)
@@ -351,7 +351,7 @@ class Gui(abstract.Gui):
         return btn
     def on_backspace(self, widget=None):
         if self.m_t.q_status == const.QSTATUS_SOLVED:
-           return
+            return
         self.g_rhythm_viewer.backspace()
         if not self.g_rhythm_viewer.m_data:
             self.g_backspace.set_sensitive(False)
